@@ -6,4 +6,4 @@
 export MACOSX_DEPLOYMENT_TARGET=10.2
 
 cd $BUILD_DIR
-make -k
\ No newline at end of file
+make
\ No newline at end of file
